//------------------------------------------------------------------------
// File and Version Information: DrcDtag.cc
// 
// temporary definition of DIRC dtag structure
//------------------------------------------------------------------------

#include "DrcDtag.hh"  //dtag format


//
//constructors
//

//use defaults
DrcDtag::DrcDtag(int doCheckCIndex, int doCheckSIndex) 
 : DrcTagMap(doCheckCIndex, doCheckSIndex), 
   _CShashTable(0), _cindexMax(_NMOD*_NSECT*_NCHIP*_NCHAN)
{ 
  //build compact hash table
  if((_CShashTable=new int[_cindexMax]) != 0) {
    for(int cindex=0;cindex<_cindexMax;cindex++) {
      _CShashTable[cindex]=CShashFunc(cindex);
    }
  }
}


//
// Hash functions used for dtags mapping...
//

//compact to sparse
int DrcDtag::CShashFunc(int cindex){
 //useful numbers
 int nchpermod=_NSECT*_NCHIP*_NCHAN;  //nb of channels per module
 int nchpersect=_NCHIP*_NCHAN;      //nb of channels per sector
 int nchperchip=_NCHAN;           //nb of channels per chip

 //indexes
 int imod=cindex/nchpermod;                             //module index (ROM)
 int isect=(cindex%nchpermod)/nchpersect;               //sector index (DFB)
 int ichip=((cindex%nchpermod)%nchpersect)/nchperchip;  //chip index  (analog chip)
 int ichan=((cindex%nchpermod)%nchpersect)%nchperchip;  //channel index (in chip)
 int idttr=_IDET;  //??

 //compute sparse index
 int sindex = ((ichan & _chanMask) << _chanShft) |
              ((ichip & _chipMask) << _chipShft) |
              ((isect & _sectMask) << _sectShft) |
              ((imod & _moduMask) << _moduShft) |
              ((idttr & _dttrMask) << _dttrShft);

 return sindex;
}

//sparse to compact
int DrcDtag::SChashFunc(int sindex){
 //useful numbers 
 int nchpermod=_NSECT*_NCHIP*_NCHAN;  //nb of channels per module
 int nchpersect=_NCHIP*_NCHAN;      //nb of channels per sector
 int nchperchip=_NCHAN;           //nb of channels per chip
 
  //decode dtag format...could use already existing functions
 int ichan=(sindex >> _chanShft) & _chanMask;
 int ichip=(sindex >> _chipShft) & _chipMask;
 int isect=(sindex >> _sectShft) & _sectMask;
 int imod=(sindex >> _moduShft) & _moduMask;
 int idet=(sindex >> _dttrShft) & _dttrMask;

 //compact index
 int cindex=imod*nchpermod+isect*nchpersect+ichip*nchperchip+ichan;
 
 return cindex;
}

//
//check functions (to be completed)
//
int DrcDtag::CheckSIndex(int sindex) const { 
  //decode dtag format...could use already existing functions
  int ichan=(sindex >> _chanShft) & _chanMask;
  int ichip=(sindex >> _chipShft) & _chipMask;
  int isect=(sindex >> _sectShft) & _sectMask;
  int imod=(sindex >> _moduShft) & _moduMask;
  int idet=(sindex >> _dttrShft) & _dttrMask;

  if( (ichan >= _NCHAN) ||
      (ichip >= _NCHIP) ||
      (isect >= _NSECT) ||
      (imod >= _NMOD) ||
      (idet != _IDET) ) return 0;
  else return 1;
}


int DrcDtag::CheckCIndex(int cindex) const { 
 static int nchdirc=_NMOD*_NSECT*_NCHIP*_NCHAN;  //nb of channels in DIRC
 if(cindex < nchdirc) return 1;
 else return 0;
}

