//------------------------------------------------------------------------
// File and Version Information: DrcGtag.cc
// 
// temporary definition of DIRC gtag structure
//------------------------------------------------------------------------

#include "DrcGtag.hh"  //gtag format

//See end of file for (big) statics initialization

//
//constructors
//

//use defaults
DrcGtag::DrcGtag(int doCheckCIndex, int doCheckSIndex) 
 : DrcTagMap(doCheckCIndex, doCheckSIndex), 
   _harnessIndexTable(_defaultHarnessIndexTable),
   _emoduleIndexTable(_defaultModuleIndexTable),
   _CShashTable(0), _cindexMax(_NMOD*_NSECT*_NCHIP*_NCHAN)
{ 
  initArrays();  
  buildChanIndex(_harnessIndexTable);
  //build compact hash table
  if((_CShashTable=new int[_cindexMax]) != 0) {
    for(int cindex=0;cindex<_cindexMax;cindex++) {
      _CShashTable[cindex]=CShashFunc(cindex);
    }
  }
}

//change defaults
DrcGtag::DrcGtag(GeomArray &HarnessIndexTable, GeomModuleArray &ModuleIndexTable, int doCheckCIndex, int doCheckSIndex)
 : DrcTagMap(doCheckCIndex, doCheckSIndex),
   _harnessIndexTable(HarnessIndexTable),
   _emoduleIndexTable(ModuleIndexTable),
   _CShashTable(0), _cindexMax(_NMOD*_NSECT*_NCHIP*_NCHAN)
{ 
  initArrays();  
  buildChanIndex(_harnessIndexTable);
  //build compact hash table
  if((_CShashTable=new int[_cindexMax]) != 0) {
    for(int cindex=0;cindex<_cindexMax;cindex++) {
      _CShashTable[cindex]=CShashFunc(cindex);
    }
  }
}


//
//utility routines
//

//general init
void DrcGtag::initArrays() {
  for(int ix=0;ix<_NGX;ix++) {
    for(int iy=0;iy<_NGY;iy++) {
      _chipIndexTable[ix][iy]=0;
      _chanIndexTable[ix][iy]=0;
    }
  }
}


//more specific...
void DrcGtag::buildChanIndex(const GeomArray &HarnessIndexTable) const {
 int nHarness[_NSECT];
  
  for(int i=0;i<_NSECT;i++) nHarness[i]=-1;   //init
 
  for(int ix=0;ix<_NGX;ix++) {
    for(int iy=0;iy<_NGY;iy++) {
      if(_harnessIndexTable[ix][iy]!=-1) {
        int ichan=++nHarness[HarnessIndexTable[ix][iy]];
        _chanIndexTable[ix][iy]=ichan%_NCHAN;
        _chipIndexTable[ix][iy]=ichan/_NCHAN;
      }
      else _chanIndexTable[ix][iy]=-1;
    }
  }

  //exceptions ?...(see Hoecker code)
}



//
// Hash functions used for gtags mapping...
//

//compact to sparse
int DrcGtag::CShashFunc(int cindex){
 //useful numbers
 int nchpermod=_NSECT*_NCHIP*_NCHAN;  //nb of channels per module
 int nchpersect=_NCHIP*_NCHAN;      //nb of channels per sector
 int nchperchip=_NCHAN;           //nb of channels per chip

 //indexes
 int imod=cindex/nchpermod;                             //module index (ROM)
 int isect=(cindex%nchpermod)/nchpersect;               //sector index (DFB)
 int ichip=((cindex%nchpermod)%nchpersect)/nchperchip;  //chip index  (analog chip)
 int ichan=((cindex%nchpermod)%nchpersect)%nchperchip;  //channel index (in chip)
 int iddtr=_IDET;  //??

 //go from dtag to gtag information  
 for(GeomSectorChanIndex ixy=0;ixy<_NGX*_NGY;ixy++) {  //index in geom sector   
   if((harness(ixy)==isect) && (chip(ixy)==ichip) && (chan(ixy)==ichan)) break;
 } 
 for(GeomSectorIndex igsector=0;igsector<_NGX*_NGY;igsector++) {  //geom sector index
   if(module(igsector)==imod) break;
 }
 
 //compute sparse geometrical index
 int sindex = ((ixy & _xyMask) << _xyShft) |
              ((igsector & _gsectorMask) << _gsectorShft);
              
 return sindex;
}

//sparse to compact
int DrcGtag::SChashFunc(int sindex){
 //useful numbers 
 int nchpermod=_NSECT*_NCHIP*_NCHAN;  //nb of channels per module
 int nchpersect=_NCHIP*_NCHAN;      //nb of channels per sector
 int nchperchip=_NCHAN;           //nb of channels per chip
 
 //decode geom sparse info
 int ixy=(sindex >> _xyShft) & _xyMask;
 int igsector=(sindex >> _gsectorShft) & _gsectorMask;

 //go from gtag to dtag information
 int ichan=chan(ixy)%_NCHAN;
 int ichip=chan(ixy)/_NCHAN;
 int isect=harness(ixy); 
 int imod=module(igsector);
 int idet=_IDET;  //??  

 
 //compute compact index
 int cindex=imod*nchpermod+isect*nchpersect+ichip*nchperchip+ichan;

 return cindex;
}

//
//check functions (to be completed)
//
int DrcGtag::CheckSIndex(int sindex) const { 
 if(sindex>0) return 1; 
 else return 0; 
}

int DrcGtag::CheckCIndex(int cindex) const { 
 if(cindex>0) return 1; 
 else return 0; 
}


//
//statics initialization
//

//dummy
const GeomModuleArray DrcGtag::_defaultModuleIndexTable= {0,1,2,3,4,5,6,7,8,9,10,11};

//not correct: should be inverted from Fortran...
//didn't have enough courage to do it yet. 
const GeomArray DrcGtag::_defaultHarnessIndexTable= {

   1, 2, 3, 4, 5, 6, 7, 8, 8, 9,10,10,-1,-1,-1,
  -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,

   1, 2, 3, 4, 5, 6, 7, 7, 8, 9, 9,10,11,-1,-1,
  -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,

   1, 2, 2, 3, 4, 5, 6, 7, 8, 8, 9,10,10,11,-1,
  -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,

   1, 2, 3, 4, 5, 6, 7, 7, 8, 9, 9,10,11,-1,-1,
  -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,

   1, 2, 2, 3, 4, 5, 6, 7, 8, 8, 9,10,10,11,-1,
  -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,

   1, 1, 2, 3, 4, 5, 6, 7, 7, 8, 9, 9,10,11,11,
  -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,

   1, 1, 2, 2, 3, 4, 5, 6, 7, 8, 8, 9,10,10,11,
  11,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,

   0, 1, 2, 3, 4, 5, 6, 7, 7, 8, 9, 9,10,11,11,
  -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,

   0, 1, 2, 2, 3, 4, 5, 6, 7, 8, 8, 9,10,10,11,
  12,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,

   0, 0, 1, 2, 3, 4, 5, 6, 7, 7, 8, 9, 9,10,11,
  11,12,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,

   0, 0, 1, 1, 2, 3, 4, 5, 6, 7, 8, 8, 9,10,10,
  11,12,12,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,

   0, 0, 1, 2, 3, 4, 5, 6, 7, 7, 8, 9, 9,10,11,
  11,12,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,

   0, 0, 1, 2, 3, 4, 5, 6, 6, 7, 8, 8, 9,10,10,
  11,11,12,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,

   0, 0, 1, 2, 3, 4, 5, 6, 6, 7, 7, 8, 9, 9,10,
  10,11,12,13,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,

   0, 0, 1, 1, 2, 3, 4, 5, 6, 6, 7, 8, 8, 9, 9,
  10,11,11,12,13,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,

   0, 0, 1, 2, 3, 4, 5, 6, 6, 7, 7, 8, 8, 9,10,
  10,11,12,13,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,

   0, 0, 1, 2, 3, 4, 5, 5, 6, 6, 7, 7, 8, 9, 9,
  10,11,11,12,13,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,

   0, 0, 1, 2, 3, 3, 4, 5, 6, 6, 7, 7, 8, 8, 9,
  10,10,11,12,12,13,-1,-1,-1,-1,-1,-1,-1,-1,-1,

   0, 0, 1, 1, 2, 3, 4, 5, 5, 6, 6, 7, 7, 8, 9,
   9,10,11,11,12,13,13,-1,-1,-1,-1,-1,-1,-1,-1,

   0, 0, 1, 2, 3, 4, 4, 5, 6, 6, 7, 7, 8, 8, 9,
  10,10,11,12,12,13,-1,-1,-1,-1,-1,-1,-1,-1,-1,

   0, 0, 1, 2, 3, 3, 4, 5, 5, 6, 6, 7, 7, 8, 9,
   9,10,11,11,12,13,13,-1,-1,-1,-1,-1,-1,-1,-1,

   0, 0, 1, 2, 2, 3, 4, 4, 5, 6, 6, 6, 7, 8, 8,

   9,10,10,11,12,12,13,13,-1,-1,-1,-1,-1,-1,-1,

   0, 0, 1, 1, 2, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8,
   9, 9,10,11,11,12,12,13,13,-1,-1,-1,-1,-1,-1,

   0, 0, 1, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 8, 8,
   9,10,10,11,12,12,13,13,-1,-1,-1,-1,-1,-1,-1,

   0, 0, 1, 2, 2, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8,
   9, 9,10,11,11,12,12,13,13,-1,-1,-1,-1,-1,-1,

   0, 0, 1, 1, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 8,
   8, 9,10,10,11,12,12,13,13,13,-1,-1,-1,-1,-1,

   0, 0, 1, 1, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7,
   8, 9, 9,10,11,11,12,12,13,13,13,-1,-1,-1,-1,

   0, 0, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 8,
   8, 9,10,10,11,12,12,13,13,13,-1,-1,-1,-1,-1,

   0, 0, 1, 1, 2, 3, 3, 4, 4, 5, 5, 5, 6, 7, 7,
   8, 9, 9,10,11,11,12,12,13,13,13,-1,-1,-1,-1,

   0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7,
   8, 8, 9,10,10,11,12,12,13,13,13,13,-1,-1,-1,

   0, 0, 1, 2, 2, 3, 3, 4, 4, 4, 5, 5, 6, 7, 7,
   8, 9, 9,10,11,11,12,12,13,13,13,-1,-1,-1,-1,

   0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7,
   8, 8, 9,10,10,11,11,12,12,13,13,13,-1,-1,-1,

   0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 4, 5, 5, 6, 7,
   7, 8, 9, 9,10,11,11,12,12,12,13,13,13,-1,-1,

   0, 1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7,
   8, 8, 9,10,10,11,11,12,12,12,13,13,-1,-1,-1,

   0, 0, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 5, 6, 7,
   7, 8, 9, 9,10,11,11,12,12,12,13,13,13,-1,-1,

   0, 0, 1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6,
   7, 8, 8, 9,10,10,11,11,12,12,12,13,13,13,-1,

   0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 5, 6, 7,
   7, 8, 9, 9,10,10,11,11,12,12,12,13,13,-1,-1,

   0, 0, 1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6,
   7, 8, 8, 9, 9,10,10,11,12,12,12,13,13,13,-1,

   0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 5, 6,
   7, 7, 8, 8, 9, 9,10,11,11,12,12,12,13,13,13,

   0, 0, 1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6,
   7, 8, 8, 9, 9,10,10,11,11,12,12,12,13,13,-1,

   0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 5, 6,
   7, 7, 8, 8, 9, 9,10,10,11,11,12,12,13,13,13 
};
