//------------------------------------------------------------------------
// File and Version Information: DrcGtag.hh
// 
// temporary definition of DIRC gtag structure
//
// WARNING!!! the HarnessIndexTable indices should be inverted
//            because of Fortran (table 1st used in fortran)
//  Also, exceptions not taken into account    
//------------------------------------------------------------------------

#ifndef DRCGTAG_HH
#define DRCGTAG_HH

#include "DrcTagMap.hh"
#include "DrcDtag.hh"    //dtag format (because compact index very
                         //close to dtag)

//gtag format
const int  _NGX(30);    //nb of pmts in x sector dimension
const int  _NGY(41);    //nb of pmts in y sector dimension
const int  _NGSECTOR(12);   //nb pf DIRC geom sectors

const unsigned int _xyMask(0x3ff);    //to get x,y field
const unsigned int _gsectorMask(0x1f);   //to get geom sector field  
const int _xyShft(0);
const int _gsectorShft(10);

//typedefs
typedef int GeomSectorChanIndex;
typedef int GeomSectorIndex;
typedef int GeomArray[_NGX][_NGY];
typedef int GeomModuleArray[_NGSECTOR];


//main class
class DrcGtag : public DrcTagMap {

 public:
  //constructors
  DrcGtag(int doCheckCIndex, int doCheckSIndex);   //use defaults
  DrcGtag(GeomArray &HarnessIndexTable, GeomModuleArray &ModuleIndexTable, int doCheckCIndex, int doCheckSIndex);
  
  //utility routines
  void initArrays();
  void buildChanIndex(const GeomArray &HarnessIndexTable) const;
  
  //accessors
  inline int module(GeomSectorIndex igsector) { return _emoduleIndexTable[igsector];};
  inline int chip(GeomSectorChanIndex ixy) { int ix=ixy/_NGY; int iy=ixy%_NGY; return _chipIndexTable[ix][iy]; }
  inline int chip(int ix, int iy) { return _chipIndexTable[ix][iy]; }
  inline int chan(GeomSectorChanIndex ixy) { int ix=ixy/_NGY; int iy=ixy%_NGY; return _chanIndexTable[ix][iy]; }
  inline int chan(int ix, int iy) { return _chanIndexTable[ix][iy]; }
  inline int harness(GeomSectorChanIndex ixy) { int ix=ixy/_NGY; int iy=ixy%_NGY; return _harnessIndexTable[ix][iy]; }
  inline int harness(int ix, int iy) { return _harnessIndexTable[ix][iy]; }
 
  //hash functions
  int CShashFunc(int cindex);   //go from sparse to compact
  //go from compact to sparse using a cache
  inline int FastCShashFunc(int cindex){ return _CShashTable[cindex]; }
  int SChashFunc(int cindex);   //go from compact to sparse
  
  //check functions
  int CheckSIndex(int sindex) const;
  int CheckCIndex(int cindex) const;
  
 private:
  //arrays derived from basic arrays
  static int _chipIndexTable[_NGX][_NGY];     //geom x,y to chip index
  static int _chanIndexTable[_NGX][_NGY];     //geom x,y to channel index (in chip)
  const GeomModuleArray &_emoduleIndexTable;
  const GeomArray &_harnessIndexTable;
  
  //
  //Default arrays for constructing others
  //
  
  //geom sector index to module index
  static const GeomModuleArray _defaultModuleIndexTable; 
      
  //array from Hoeker (WARNING: indices inverted because of fortran !)
  static const GeomArray _defaultHarnessIndexTable;
  
  //cache
  int *_CShashTable;  //compact to sparse cache hash table
  int _cindexMax;   //max compact index value
  
};


#endif
