//------------------------------------------------------------------------
// File and Version Information:
//      $Id: ??? $
//
// Description:
//      Class DrcTagMap. Service class implementing the relations
//      between various tag maps (dtag,gtag, ptag?)
//      
// Environment:
//      Software developed for the BaBar Detector at the SLAC B-Factory.
//
// Author List:
//      ???              
//
// Copyright Information:
//      Copyright (C) 1997      Lawrence Berkeley Laboratory
//
//------------------------------------------------------------------------

#ifndef DRCTAGMAP_HH
#define DRCTAGMAP_HH


//base mapping class (1 map only, 2 way)
class DrcTagMap {
 public:
  //constructors
  DrcTagMap( );  
  DrcTagMap(int doCheckCIndex, int doCheckSIndex);   //main constructor  
  DrcTagMap(const DrcTagMap&);  //copy constructor

  //destructor
  virtual ~DrcTagMap();

  //mapping functions
  int csmap(int cindex);   //compact to sparse index
  int scmap(int sindex);   //sparse to compact index

  //hash functions (cannot be pure virtual because eventually used in constructor...)
  virtual int CShashFunc(int)=0;              //compact to spare mapping
  virtual inline int FastCShashFunc(int)=0;   //cached compact to spare mapping
  virtual int SChashFunc(int)=0;   //spare to compact mapping

  //checking functions
  virtual int CheckCIndex(int) const=0;    //check compact index (1=OK)
  virtual int CheckSIndex(int) const=0;    //check sparse index (1=OK)
  
  //utilities
//  int autotest();    //internal coherence test (easy to implement)

 private:
  int *_CShashTable;  //compact to sparse hash table
  int _cindexMax;   //max compact index value
  int _doCheckCIndex;   //check the indexes (1=do it)
  int _doCheckSIndex;   //
};

#endif
